package org.jasig.portal.plexus.components;

import java.io.File;
import java.io.IOException;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.pluto.util.UtilityException;
import org.apache.pluto.util.assemble.Assembler;
import org.apache.pluto.util.assemble.AssemblerConfig;
import org.apache.pluto.util.assemble.AssemblerFactory;

/**
 * Encapsulates utilities used for assembling plain old war files
 * suitable for deployment to a JSR-168 portlet container.
 * 
 * @author Elliot Metsger (emetsger@jhu.edu)
 * @version $Id: AssemblingUtils.java 11963 2007-04-20 21:10:29Z esm $
 * @since Mar 15, 2007
 */
public class AssemblingUtils
{
    /**
     * Logger.
     * @todo Log should be injected or otherwise obtained from Plexus or Maven.
     */
    private static final Log LOG = LogFactory.getLog(AssemblingUtils.class);
    
    /**
     * Delegates to {@link assemble#(File source, File dest)}
     * using a <code>null</code> destination.
     * <p>
     * The source file will be assembled and replaced with
     * an assembled version of the file.
     * 
     * @param source the file to assemble. 
     * @throws IOException if the assembly fails
     */
    public void assemble(File source) throws IOException
    {
        assemble(source, null);
    }
    
    /**
     * The source file is assembled to the destination directory
     * with the same file name as the source file.
     * <p>
     * If the destination is null, the source file is 
     * replaced with an assembled version of the file.
     * 
     * @param source the file to assemble
     * @param dest the directory the assembled file will be placed in (may be null)
     * @throws IOException if the assembly fails
     */
    public void assemble(File source, File dest) throws IOException
    {       
        try 
        {
            validate(source);
        } 
        catch (IOException e)
        {
            LOG.error("Unable to validate archive file: " + e.getMessage(), e);
            throw new IOException("Unable to validate archive file: " + e.getMessage());
        }
        
        AssemblerConfig config = new AssemblerConfig();
        config.setWarSource(source);
        if (dest != null)
        {
            config.setDestination(dest);
        }
        Assembler assembler = getAssembler(source, dest, config);
        
        try
        {
            String msg = "Assembling " + source.getAbsolutePath() + 
                ((dest != null) ? " to " + dest.getAbsolutePath() : null);
            LOG.debug(msg);         
            assembler.assemble(config);
        }
        catch (UtilityException e)
        {
            String msg = "Error encountered while assembling " + source.getAbsolutePath() +
                ((dest != null) ? " to " + dest.getAbsolutePath() + " : " : null ) + e.getMessage();
            LOG.error(msg, e);
            throw new IOException(msg);
        }
    }
    
    /**
     * Obtain a configured Pluto assembler.  Note the
     * destination could be null, indicating that the
     * source file should be assembled inline.
     * 
     * @return the assembler
     * @todo inject the assembler.
     */
    protected Assembler getAssembler(final File source, final File dest, final AssemblerConfig config)
    {                          
        return AssemblerFactory.getFactory().createAssembler(config);
    }
    
    /**
     * Ensures the source file is viable for assembly.
     * 
     * @param toAssemble the file being assembled
     * @throws IOException if the validation fails
     */
    protected void validate(final File toAssemble) throws IOException
    {        
        // The file should 
        //  1) already exist, 
        //  2) be readable, 
        //  3) and be writable (in case we are rewriting the file in the same directory)
        //  4) and end with ".war" extention
                
        if (toAssemble == null)
        {
            throw new IOException("Cannot assemble null file.");
        }
        
        String ePrefix = "Error validating file " + toAssemble.getAbsolutePath();
        
        if (! toAssemble.exists())
        {
            throw new IOException(ePrefix + ": File does not exist.");
        }
        
        if (! toAssemble.canRead())
        {
            throw new IOException(ePrefix + ": File cannot be read.");
        }
        
        if (! toAssemble.canWrite())
        {
            throw new IOException(ePrefix + ": File cannot be written to.");
        }
        
        // Right now we only work with war files        
        if (! toAssemble.getName().endsWith(".war"))
        {
            // If you want "fall-through" behavior - that is proceed with
            // normal archiver process without assembling - the 
            // assemblyRequired flag should be set to false.
            throw new IOException(ePrefix + ": Cannot assemble " + toAssemble.getAbsolutePath() + ", it is not a WAR.");
        }
    }
}
