package org.jasig.portal.plexus.components;

import java.io.File;
import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import org.codehaus.plexus.archiver.ArchiveEntry;
import org.codehaus.plexus.archiver.ArchiverException;
import org.codehaus.plexus.archiver.jar.JarArchiver;
import org.codehaus.plexus.logging.Logger;

/**
 * A Plexus <code>JarArchiver</code> which supports assembling 
 * (modifying the WEB-INF/web.xml suitable for deployment
 * in a JSR-168 portlet container) war files.
 * 
 * @author Elliot Metsger (emetsger@jhu.edu)
 * @version $Id: AssemblingWarArchiver.java 11911 2007-03-23 15:16:18Z edalquist $
 * @since Mar 15, 2007
 * @plexus.component role="org.codehaus.plexus.archiver.Archiver" 
 *                   role-hint="jar"
 *                   instantiation-strategy="per-lookup" 
 */
public class AssemblingWarArchiver extends JarArchiver
{
    
    /**
     * Flag indicating whether or not to assemble
     * the archive.  Defaults to true.
     * 
     * @plexus.configuration default-value="true"
     */
    private boolean assemblyRequired = true; 
        
    /**
     * This implementation calls the super class addDirectory(File) method, 
     * then obtains a Map containing the <code>ArchiveEntry</code> objects 
     * from the super class.
     * <p>
     * The ArchiveEntry objects are then converted to files, assembled,
     * and then replaced into the Map.  Note that this implementation relies upon
     * the super class <em>not</em> performing a defensive copy of the Map 
     * containing the <code>ArchiveEntry</code> objects.
     * <p>
     * If the assemblyRequired flag is false, this method immediatly returns and
     * performs no further processing.
     * 
     * @throws ArchiverException if assembly fails.
     */
    public void addDirectory(File directory) throws ArchiverException
    {
        Logger log = getLogger();
        System.out.println("Logger: [" + log.getClass().getName() + "]");
        
        // call the superclass impl
        super.addDirectory(directory);
        
        // just return if we are not assembling.
        if (! isAssemblyRequired())
        {
            log.debug("No files were selected for assembly.  Assembly not required.");
            return;
        }
        
        // utilities for performing the assembly.
        AssemblingUtils assemblyUtils = new AssemblingUtils();
        
        // The map of files this archiver is responsible for.
        // Keys are String file names - just the name of the file, not the full path.
        // Values are ArchiveEntry objects.
        Map fileMap = getFilesInternal();
        
        if (fileMap == null || fileMap.isEmpty())
        {
            log.debug("No files were selected for assembly.");
        }
        
        // Iterate over the files the archiver is responsible for,
        // saving references to the WAR ArchiveEntry objects in 
        // a new Map.
        Set entries = fileMap.entrySet();
        Map warArchiveEntries = new HashMap();
        for (Iterator i = entries.iterator(); i.hasNext();)
        {
            Map.Entry entry = (Map.Entry)i.next();           
            ArchiveEntry archiveEntry =  (ArchiveEntry)entry.getValue();
            if (archiveEntry.getName().endsWith(".war"))
            {
                log.debug("File " + archiveEntry.getFile().getAbsolutePath() + " selected for assembly.");
                warArchiveEntries.put((String)entry.getKey(), archiveEntry);
            }                
        }
        
        // if there were no war entries, just return
        if (warArchiveEntries.isEmpty())
        {
            log.debug("No files were selected for assembly.");
            return;
        }       
        
        // For each WAR ArchiveEntry:
        //  1) Obtain the original file.
        //  2) Assemble the original file to a new file.
        //  3) Create a new ArchiveEntry with the new file
        //  4) Place the new ArchiveEntry into the original map obtained with getFiles()
        Set warEntries = warArchiveEntries.entrySet();
        for (Iterator i = warEntries.iterator(); i.hasNext();)
        {
            Map.Entry entry = (Map.Entry)i.next();
            // the original filename, which is a key into the filesMap
            String fileName = (String)entry.getKey();
            // the original ArchiveEntry, which we want to assemble
            ArchiveEntry warArchive = (ArchiveEntry)entry.getValue();
            if (warArchive.getFile() == null)
            {
                // this would be highly unusual.
                log.debug("No war archive was found for file name " + fileName);
                continue;
            }
            
            // obtain the file to assemble (the source file)
            File toAssemble = warArchive.getFile();
            
            // the directory for the assembled war file to 
            // placed in
            File destDir = new File(System.getProperty("java.io.tmpdir"));
            
            // The resultant assembled file
            File assembledFile = null;
            
            // perform the assembly
            
            try
            {
                assemblyUtils.assemble(toAssemble, destDir);
                assembledFile = new File(destDir, toAssemble.getName());
                if (assembledFile == null)
                {
                    String msg = "The assembled file was expected to be in " +
                        destDir.getAbsolutePath() + File.separator + toAssemble.getName() +
                        " but it was not found.";
                    log.error(msg, new NullPointerException());
                    throw new IOException(msg);
                }
            }
            catch (IOException e)
            {
                String msg = "Unable to perform the assembly.";
                log.error(msg, e);
                throw new ArchiverException(msg, e);
            }
            
            // Create a new ArchiveEntry with permissions, 
            // but with the newly assembled file and name.
            ArchiveEntry assembledEntry = ArchiveEntry.
                createFileEntry(assembledFile.getName(), assembledFile, warArchive.getMode());
            
            // Replace the non-assembled entry with the assembled entry in the
            // fileMap
            if (fileMap.containsKey(fileName))
            {
                ArchiveEntry orig = (ArchiveEntry)fileMap.get(fileName);
                log.debug("Replacing original file map entry: key [" + fileName + "], value [" + orig + "]" +
                        " with: key [" + fileName + "] value [" + assembledEntry + "]");
                fileMap.put(fileName, assembledEntry);
            }                        
        }                
    }
    
    /**
     * Contains implementation specific logic to ensure that the 
     * <code>getFiles()</code> method of the super class returns
     * a bare (not defensivly copied) map.
     * 
     * @return the bare map
     */
    final protected Map getFilesInternal()
    {
        // the method is protected for unit testing and not meant to be extended.
        
        // Obtain the files that this archiver is responsible for.
        // fileMap keys are String file names
        // fileMap values are ArchiveEntry objects.
        
        // If my dog was this ugly....
        //  getFiles() performs a defensive copy if setIncludeEmptyDirs(false) is NOT
        //  called.  We need a reference to the bare Map, because we need to mutate
        //  its state in the superclass.  So therefore we call setIncludeEmptyDirs(false)
        //  prior to calling getFiles().
        // FIXME: override getFiles() to always return the bare Map.
        boolean originalEmptyDirFlag = getIncludeEmptyDirs();
        setIncludeEmptyDirs(false);
        
        Map fileMap = getFiles();
        setIncludeEmptyDirs(originalEmptyDirFlag);
        return fileMap;
    }

    public boolean isAssemblyRequired()
    {
        return assemblyRequired;
    }

    public void setAssemblyRequired(boolean assemblyRequired)
    {
        this.assemblyRequired = assemblyRequired;
    }
}
